/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.api;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.Cloneable;

/**
 * A Field is one of the components of a TupleStructure.
 * @author Sebastien Chassande-Barrioz
 * @see TupleStructure
 */
public interface Field extends Cloneable {
    //The name of a field of type PName (NameDef) for extents
    final static String PNAMENAME = "_PName";

	//variables used to manage the null status of the Field
    public static final short NONULLS = 1;
    public static final short NULLABLE = 2;
    public static final short NULLABLEUNKNOWN = 3;

	/**
	 * Returns the Field name
	 * @return the Field name
	 */
    public String getName();

	/**
	 * Returns the field type, as a PType
	 * @return the Field type
	 */
    public PType getType();

    /**
     * What is the status of the Field regarding null values (NullStatus).
	 * @return the Null status of the Field
	 * @see NullStatus
     */
	//TODO remove this method if unused
    public short getNullStatus();
}
