/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.datasource.api;

/**
 * Describes data stores, i.e., entities storing data, such as a relational
 * database, an object-oriented database, an LDAP directory, a UDDI directory,
 * an XML document store, a text file storage system, etc.
 */
public interface DataStore {

    public static final short MEDORTC_STORE = 0;
    public static final short JDBC_STORE = 1;
    public static final short TEXTFILE_STORE = 2;
    public static final short ODMG_STORE = 3;
    public static final short XMLDOCUMENT_STORE = 4;
	public static final short UDDI_STORE = 5;

	/**
	 * Returns the symbolic name of the DataStore.
	 * <p>This name is used to
	 * pass as a parameter the connection to use at evaluation time for
	 * all QueryLeaves attached to this DataStore.
	 * @return the symbolic name of the current DataStore.
	 */
	public String getName();

	/**
	 * Checks whether the current DataStore object represents the same data
	 * store as another DataStore object.
	 * @param ds the other DataStore object
	 * @return true of the two DataStore objects represent the same data store.
	 */
	public boolean isSameAs(DataStore ds);

	/**
	 * Checks whether the current DataStore is capable of performing different
	 * types of operations.
	 * @param operationType is the type of operation for checking.
	 * @return true if the current DataStore is capable of performing the input
	 * operation type
	 * @see org.objectweb.medor.query.api.OperationType
	 */
	public boolean isCapable(short operationType);

	/**
	 * Returns the type of DataStore
	 * @return the type of DataStore
	 */
    public short getDataStoreType();
}
