/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.datasource.api;

import org.objectweb.medor.api.EvaluationException;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.tuple.api.TupleCollection;

/**
 * Represents a wrapper to access data from a QueryLeaf at query evaluation
 * time.
 * <p>A Wrapper is associated to a QueryLeaf at a given point of time.
 * There can be several Wrappers of the same type.
 * One Wrapper can be associated sequentially to several QueryLeaves.
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface Wrapper {
	/**
	 * This method links the wrapper to a particular QueryLeaf, and executes
	 * the query on the underlying data store.
	 * @param ql the QueryLeaf
	 * @param parameters the parameters necessary for obtaining data from the
	 * QueryLeaf (in the case the QueryLeaf contains an expression)
	 * @param connection an Object which contains information for connecting
	 * to the DataStore behind the QueryLeaf (could be a JDBC connection, ...)s
	 * @return the TupleCollection containing the result of the execution.
	 * @throws EvaluationException in case of problems during the evaluation.
	 *
	 */
	public TupleCollection fetchData(QueryLeaf ql, ParameterOperand[] parameters,
							  Object connection)
		throws EvaluationException;
}
