/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.cache.lib;

/**
 * This class
 */
import org.objectweb.medor.eval.cache.api.CollectionCache;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.api.MedorException;

import java.util.Hashtable;
import java.util.Iterator;

public class TupleCache implements CollectionCache {

    public TupleCache(long size) {
        capacity = size;
        cache = new Hashtable();
    }

    private long capacity;
    private int smallIndex = 1;
    private boolean canInsert = true;
    private Hashtable cache;

    public synchronized long getCapacity() {
        return capacity;
    }

    public synchronized void setCapacity(int capacity) {
        this.capacity = capacity;
    }

    public synchronized Tuple getTuple(int index) throws MedorException {
        Tuple t = null;
        if (cache.containsKey(new Integer(index))) {
            t = (Tuple) cache.get(new Integer(index));
        }
        return t;
    }

    public synchronized boolean putTuple(int index, Tuple t) throws MedorException {
        boolean isPuted = false;
        if (canInsert) {
            if (!cache.containsKey(new Integer(index))) {
                if (cache.size() >= capacity) {
                    cache.remove(new Integer(smallIndex));
                    smallIndex++;
                }
                cache.put(new Integer(index), t); // Make sure that t is cloned and not a reference
                isPuted = true;
            } //else
 				//System.out.println("Not puted in the cache because already existe:  " + index);
        } //else
            //System.out.println("Not puted in the cache because cache full:  " + index);
        return isPuted;
    }

    public synchronized void initialize() {
    }

    public synchronized Iterator tupleIndexIterator() {
        return cache.keySet().iterator();
    }

    public synchronized void destroy() {
        cache.clear();
    }

    public synchronized boolean contains(int index) {
        return cache.containsKey(new Integer(index));
    }

    public boolean isCanInsert() {
        return canInsert;
    }

    public void setCanInsert(boolean canInsert) {
        this.canInsert = canInsert;
    }
}
