/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.eval.prefetch.api;

import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.api.MedorException;

/**
 * Registers the prefetched tuples within an indexed buffer for further access
 * through the tuple identifier used for indexing. This registering occurs while
 * tuple collection they belong to is traversed by the application.
 * @author P. Dechamboux
 */
public interface PrefetchBuffer {
    /**
     * Adds to this cache a prefetched tuple that is the one which currently
     * traversed within the associated tuple collection.
     */
    void addPrefetchTuple() throws MedorException;

    /**
     * Asscoiates a tuple collection to this PrefetchBuffer. It should contain
     * the tuples that will be registered with the prefetch structure.
     * @param tc    The associated tuple collection.
     */
    void setTupleCollection(TupleCollection tc) throws MedorException;
}
