/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.eval.prefetch.lib;

import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.api.MedorException;

import java.util.HashMap;

/**
 * @author  P. Dechamboux
 */
public class PrefetchBufferImpl implements PrefetchBuffer {
    private int indexPos;
    private TupleCollection tupleCollection = null;
    private HashMap tupleMap = null;

    PrefetchBufferImpl(int indexpos) {
        indexPos = indexpos;
    }

    Tuple getTuple(Object index) throws MedorException {
        if (tupleMap == null) {
            return null;
        }
        Integer i = (Integer) tupleMap.get(index);
        if (i == null) {
            return null;
        }
        int row = i.intValue();
        int oldrow = -1;
        Tuple res = null;
        try {
            oldrow  = tupleCollection.getRow();
            if (oldrow != row) {
                tupleCollection.row(row);
            }
            res = tupleCollection.getTuple();
        } finally {
            // try to restore old position into tupleCollection
            if (oldrow != row) {
                try {
                    tupleCollection.row(oldrow);
                } catch (MedorException e) {
                    // ARRGH: cannot do anything more!!
                }
            }
        }
        tupleMap.remove(index);
        return res;
    }

    void close() {
        tupleCollection = null;
        tupleMap = null;
    }

    // IMPLEMENTATION OF METHODS FROM THE PrefetchCache INTERFACE

    public void addPrefetchTuple() throws MedorException {
        if (tupleMap == null) {
            tupleMap = new HashMap();
        }
        Tuple pt = tupleCollection.getTuple();
        tupleMap.put(pt.getObject(indexPos), new Integer(tupleCollection.getRow()));
    }

    public void setTupleCollection(TupleCollection tc) throws MedorException {
        if (tupleCollection != null) {
            throw new MedorException("A tuple collection has already been assigned to this PrefetchBuffer!!");
        }
        tupleCollection = tc;
    }
}
