/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.eval.prefetch.lib;

import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.api.MedorException;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author  P. Dechamboux
 */
public class PrefetchCacheImpl implements PrefetchCache {
    private HashMap txContexts = new HashMap();

    // IMPLEMENTATION OF METHODS FROM THE PrefetchCache INTERFACE

    public Tuple getPrefetchTuple(Object index, Object cat, Object ctxt)
            throws MedorException {
        if (ctxt == null) {
            throw new MedorException("Cannot search a prefetch tuple without a transaction context!");
        }
        if (cat == null) {
            throw new MedorException("Cannot search a prefetch tuple without an object category!");
        }
        HashMap catmap = (HashMap) txContexts.get(ctxt);
        if (catmap == null) {
            return null;
        }
        ArrayList pblist = (ArrayList) catmap.get(cat);
        if (pblist == null) {
            return null;
        }
        Tuple res = null, tmp;
        for(int i=0; i<pblist.size(); i++) {
            PrefetchBufferImpl pb = (PrefetchBufferImpl) pblist.get(i);
            tmp = pb.getTuple(index);
            if (res == null && tmp != null) {
                res = tmp;
            }
        }
        return res;
    }

    public void invalidatePrefetchBuffer(Object ctxt) {
        HashMap catmap = (HashMap) txContexts.get(ctxt);
        if (catmap == null) {
            return;
        }
        txContexts.remove(ctxt);
        Iterator it = catmap.values().iterator();
        while (it.hasNext()) {
            ArrayList pblist = (ArrayList) it.next();
            Iterator it2 = pblist.iterator();
            while (it2.hasNext()) {
                PrefetchBufferImpl pb = (PrefetchBufferImpl) it2.next();
                pb.close();
            }
        }
    }

    public synchronized PrefetchBuffer createPrefetchBuffer(Object cat, Object ctxt, int indexpos)
            throws MedorException {
        if (ctxt == null) {
            throw new MedorException("Cannot create a prefetch buffer without a transaction context!");
        }
        if (cat == null) {
            throw new MedorException("Cannot create a prefetch buffer without an object category!");
        }
        HashMap catmap = (HashMap) txContexts.get(ctxt);
        if (catmap == null) {
            catmap = new HashMap();
            txContexts.put(ctxt, catmap);
        }
        ArrayList pblist = (ArrayList) catmap.get(cat);
        if (pblist == null) {
            pblist = new ArrayList();
            catmap.put(cat, pblist);
        }
        PrefetchBuffer res = new PrefetchBufferImpl(indexpos);
        pblist.add(res);
        return res;
    }
}
