/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.api;

import org.objectweb.medor.api.TypingException;

import java.sql.Timestamp;
import java.sql.Time;
import java.util.Date;

/**
 * This interface represents a binary arithmetic operation (+, -, ...).
 * <p>
 * It gives methods for a binary evaluation with all possible cases.
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface BinaryArithmeticOperator extends BinaryOperator {

	/**
	 * This fonction evaluate the arithmetic operation between two numerics
	 * variables;
	 * @param op1 integer
	 * @param op2 integer
	 * @return an integer result
	 */
	int evaluate(int op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 short
	 * @return an int as in Java programming language
	 */
	int evaluate(int op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 long
	 * @return a long result as in Java programming language
	 */
	long evaluate(int op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 float
	 * @return a float result as in Java programming language
	 */
	float evaluate(int op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 double
	 * @return a double as in Java programming language
	 */
	double evaluate(int op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two floats variables;
	 * @param op1 float
	 * @param op2 float
	 * @return a float resultas in Java programming language
	 */
	float evaluate(float op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 short
	 * @return a float result as in Java programming language
	 */
	float evaluate(float op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 int
	 * @return a float result as in Java programming language
	 */
	float evaluate(float op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 long
	 * @return a float result as in Java programming language
	 */
	float evaluate(float op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 double
	 * @return a double result as in Java programming language
	 */
	double evaluate(float op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two characters variable;
	 * @param op1 character
	 * @param op2 character
	 * @return an int value
	 */
	int evaluate(char op1, char op2);

	/**
	 * This fonction evaluate the comaparison between two characters variable;
	 * @param op1 char
	 * @param op2 string
	 * @return a string value
	 * @throws TypingException if this operation is not supported
	 */
	String evaluate(char op1, String op2)throws TypingException;


	/**
	 * This fonction evaluate the comaparison between two long variable;
	 * @param op1 long
	 * @param op2 long
	 * @return a long result as in Java programming language
	 */
	long evaluate(long op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 short
	 * @return a long result as in Java programming language
	 */
	long evaluate(long op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 int
	 * @return a long result as in Java programming language
	 */
	long evaluate(long op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 float
	 * @return a float result as in Java programming language
	 */
	float evaluate(long op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 double
	 * @return a double result as in Java programming language
	 */
	double evaluate(long op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two double variable;
	 * @param op1 double
	 * @param op2 double
	 * @return a double result as in Java programming language
	 */
	double evaluate(double op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 short
	 * @return a double result as in Java programming language
	 */
	double evaluate(double op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 int
	 * @return a double result as in Java programming language
	 */
	double evaluate(double op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 float
	 * @return a double result as in Java programming language
	 */
	double evaluate(double op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 long
	 * @return a double result as in Java programming language
	 */
	double evaluate(double op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two string variable;
	 * @param op1 String
	 * @param op2 String
	 * @return a string value as in java programming language
	 * @throws TypingException if this operation is not supported
	 */
	String evaluate(String op1, String op2)throws TypingException;

	/**
	 * This fonction evaluate the comaparison between two string variable;
	 * @param op1 String
	 * @param op2 char
	 * @return a string value as in java programming language
	 * @throws TypingException if this operation is not supported
	 */
	String evaluate(String op1, char op2)throws TypingException;

	/**
	 * This fonction evaluate the comaparison between two java.sql.Timestamp
	 * variable;
	 * @param op1 TimeStamp
	 * @param op2 Timestamp
	 * @return a Timestamp value as in java programming language
	 * @throws TypingException if this operation is not supported
	 */
	Timestamp evaluate(Timestamp op1, Timestamp op2)throws TypingException;

	/**
	 * This fonction evaluate the comaparison between two java.sql.Time variable;
	 * @param op1 Time
	 * @param op2 Time
	 * @return a Time value as in java programming language
	 * @throws TypingException if this operation is not supported
	 */
	Time evaluate(Time op1, Time op2)throws TypingException;

	/**
	 * This fonction evaluate the comaparison between two java.util.Date variable;
	 * @param op1 Date
	 * @param op2 Date
	 * @return a Date value as in java programming language
	 * @throws TypingException if this operation is not supported
	 */
	Date evaluate(Date op1, Date op2)throws TypingException;
}
