/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.api;

/**
 * This interface represents a mathematical comparator. Comparators are:
 * > < <= >= !=. the 'evaluate' methods do the comparison betwwen
 * different type of variables: int, char, boolean,...
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface Comparator extends BinaryOperator {

	/**
	 * This fonction evaluate the comaparison between two java Object ;
	 * @param op1 Object
	 * @param op2 Object
	 * @return true if the comparison is verified
	 */
	boolean evaluate(Object op1, Object op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 int
	 * @return true if the comparison is verified
	 */
	boolean evaluate(int op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 short
	 * @return true if the comparison is verified
	 */
	boolean evaluate(int op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 long
	 * @return true if the comparison is verified
	 */
	boolean evaluate(int op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 float
	 * @return true if the comparison is verified
	 */
	boolean evaluate(int op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 int
	 * @param op2 double
	 * @return true if the comparison is verified
	 */
	boolean evaluate(int op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two floats variables;
	 * @param op1 float
	 * @param op2 float
	 * @return true if the comparison is verified
	 */
	boolean evaluate(float op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 short
	 * @return true if the comparison is verified
	 */
	boolean evaluate(float op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 int
	 * @return true if the comparison is verified
	 */
	boolean evaluate(float op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 long
	 * @return true if the comparison is verified
	 */
	boolean evaluate(float op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 float
	 * @param op2 double
	 * @return true if the comparison is verified
	 */
	boolean evaluate(float op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two characters variable;
	 * @param op1 char
	 * @param op2 char
	 * @return true if the comparison is verified
	 */
	boolean evaluate(char op1, char op2);

	/**
	 * This fonction evaluate the comaparison between two long variable;
	 * @param op1 long
	 * @param op2 long
	 * @return true if the comparison is verified
	 */
	boolean evaluate(long op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 short
	 * @return true if the comparison is verified
	 */
	boolean evaluate(long op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 int
	 * @return true if the comparison is verified
	 */
	boolean evaluate(long op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 float
	 * @return true if the comparison is verified
	 */
	boolean evaluate(long op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 long
	 * @param op2 double
	 * @return true if the comparison is verified
	 */
	boolean evaluate(long op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two double variable;
	 * @param op1 double
	 * @param op2 double
	 * @return true if the comparison is verified
	 */
	boolean evaluate(double op1, double op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 short
	 * @return true if the comparison is verified
	 */
	boolean evaluate(double op1, short op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 int
	 * @return true if the comparison is verified
	 */
	boolean evaluate(double op1, int op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 float
	 * @return true if the comparison is verified
	 */
	boolean evaluate(double op1, float op2);

	/**
	 * This fonction evaluate the comaparison between two numerics variables;
	 * @param op1 double
	 * @param op2 long
	 * @return true if the comparison is verified
	 */
	boolean evaluate(double op1, long op2);

	/**
	 * This fonction evaluate the comaparison between two string variable;
	 * @param op1 String
	 * @param op2 String
	 * @return true if the comparison is verified
	 */
	boolean evaluate(String op1, String op2);

	/**
	 * This fonction evaluate the comaparison between two boolean variable;
	 * @param op1 boolean
	 * @param op2 boolean
	 * @return true if the comparison is verified; in case of <, > compartor
     * it always return false
	 */
	boolean evaluate(boolean op1, boolean op2);

}
