/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.api;

import org.objectweb.medor.api.Cloneable;
import org.objectweb.medor.api.ExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.tuple.api.Tuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface ExpressionHelper extends Cloneable {

    public Expression[] toStack(Expression e)
            throws ExpressionException, TypingException;

    public Expression join(Expression[] es)
            throws ExpressionException, TypingException;

    public Operand evaluate(Expression[] stack, ParameterOperand[] parameters,
                            Tuple t)
            throws MedorException, IllegalStateException;

    public Expression substituteParameters(ParameterOperand[] pos,
                                           Expression e)
            throws ExpressionException, TypingException;

    public Expression[] substituteParameters(ParameterOperand[] pos,
                                             Expression[] e)
            throws ExpressionException, TypingException;

    public String[] getParameterNames(Expression e) throws ExpressionException,
            TypingException;

    /**
     * For a given Expression used at a QueryNode, sets the index of its
     * FieldOperands (index in the tuple resulting from the concatenation
     * of the children QueryNode tuples).
     * It will call the setIndex method of
     * <a href="FieldOperand.html">FieldOperand</a>.
     * @param node: the QueryNode to which the Expression is attached
     * @param e: the Expression to be visited for setting the index of its
     * FieldOperands.
     * @see org.objectweb.medor.filter.api.FieldOperand
     */
    public void fixIndexes(QueryNode node, Expression e);
}
