/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.filter.api;

import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.jorm.type.api.PType;

import java.util.Date;
import java.sql.Time;
import java.sql.Timestamp;

/**
 * This Interface extends a simple operand. It offers methods to sets new value
 * to this object. The type of this operand does not change.
 */
public interface VariableOperand extends Operand {

    /**
     *  This method changes the type of the current operand object.
     */
    void setType(PType type);

    /**
     * It assigns the operand value as a boolean value
     * @param p the new boolean value.
     */
    void setValue(boolean p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new integer value.
     */
    void setValue(int p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new byte value.
     */
    void setValue(byte p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new short value.
     */
    void setValue(short p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new long value.
     */
    void setValue(long p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new float value.
     */
    void setValue(float p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new double value.
     */
    void setValue(double p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new char value.
     */
    void setValue(char p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new String value.
     */
    void setValue(String p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new java.util.Date value.
     */
    void setValue(Date p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new java.sql.Time value.
     */
    void setValue(Time p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new java.sql.Timestamp value.
     */
    void setValue(Timestamp p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new Tuple value.
     */
    void setValue(Tuple p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new TupleCollection value.
     */
    void setValue(TupleCollection p) throws TypingException;

    /**
     * It assigns the operand value
     * @param p the new Object value.
     */
    void setValue(Object p) throws TypingException;
}
