/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.BelongOperator;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicBelongOperator
        extends BasicBinaryOperator
        implements BelongOperator {

    public void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException, IllegalStateException {
        try {
            left.evaluate(pos, t);
            right.evaluate(pos, t);
            result.setValue(evaluate(left.getResult().getTuple(),
							right.getResult().getTupleCollection()));
        } catch (NullPointerException e) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
    }

    public boolean evaluate(Tuple tuple, TupleCollection tc)
            throws MedorException {
        Tuple element;
        boolean belong = false;
        if (tc.isEmpty())
            belong = false;
        else {
            do {
                element = tc.getTuple();
                belong = equalTuple(element, tuple);
            } while ((!belong) && (tc.next()));
        }
        return belong;
    }

    private boolean equalTuple(Tuple t1, Tuple t2) throws MedorException {
        boolean equalOperand = true;
        if (t1.getSize() == t2.getSize()) {
            int cpt = 1;
            while ((cpt < t1.getSize()) && (equalOperand)) {
                if (t1.getLikeOperand(cpt).getType() ==
					t1.getLikeOperand(cpt).getType()) {
                    switch (t1.getLikeOperand(cpt).getType().getTypeCode()) {
                        case QTypeTuple.TYPECODE_INT:
                            equalOperand = (t1.getInt(cpt) == t2.getInt(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_SHORT:
                            equalOperand =
								(t1.getShort(cpt) == t2.getShort(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_BYTE:
                            equalOperand = (t1.getByte(cpt) == t2.getByte(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_LONG:
                            equalOperand = (t1.getLong(cpt) == t2.getLong(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_DOUBLE:
                            equalOperand =
								(t1.getDouble(cpt) == t2.getDouble(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_BOOLEAN:
                            equalOperand =
								(t1.getBoolean(cpt) == t2.getBoolean(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_FLOAT:
                            equalOperand =
								(t1.getFloat(cpt) == t2.getFloat(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_STRING:
                            equalOperand =
								(t1.getString(cpt).equalsIgnoreCase(t2.getString(cpt)));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_CHAR:
                            equalOperand = (t1.getChar(cpt) == t2.getChar(cpt));
                            cpt++;
                            break;
                        case QTypeTuple.TYPECODE_DATE:
                            equalOperand =
								((t1.getDate(cpt).compareTo(t2.getDate(cpt)) == 0));
                            cpt++;
                            break;

/*
                            case QTypeTuple.TYPECODE_TIME:
                                equalOperand = ((t1.getTime(cpt).compareTo(t2.getTime(cpt)) == 0));
                                cpt++;
                                break;
                            case QTypeTuple.TYPECODE_TIMESTAMP:
                                equalOperand = ((t1.getTimestamp(cpt).compareTo(t2.getTimestamp(cpt)) == 0));
                                cpt++;
                                break;
                                */
                    }
                }
            }
        }
        return equalOperand;
    }

    public void compileExpression()
		throws TypingException, MalformedExpressionException {
        if (!verified) {
            if ((left == null) || (right == null))
                throw new MalformedExpressionException("null children value");
            else {
                left.compileExpression();
                leftResult = left.getResult();
                right.compileExpression();
                rightResult = right.getResult();
                //if ((left.getType() == "tuple") && (right.getType() == "TC")){
				// TO BE IMPLEMENTED CORRECTLY
                type = PTypeSpaceMedor.BOOLEAN;
                verified = true;
                result = new BasicVariableOperand(PTypeSpaceMedor.BOOLEAN);
				// Be carful, we don't create an operand result at each
                //}else throw new TypingException("incopatible expression types")
            }
        }
    }

	/**
	 * There is no Java operator for belong.
	 */
	public String getOperatorString() {
		return null;
	}
}
