/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.BinaryOperator;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.VariableOperand;
import org.objectweb.medor.tuple.api.Tuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicBinaryOperator implements BinaryOperator {

	protected PType type = null;
	protected Expression left = null;
	protected Expression right = null;
	protected VariableOperand result = null;
	protected Operand leftResult = null, rightResult = null;
	protected boolean verified = false;

    // IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

    public int getOperandNumber() {
        return 2;
    }

    public Expression getExpression(int idx)
        throws ArrayIndexOutOfBoundsException {
        if (idx == 0)
            return getLeftExpression();
        else if (idx == 1)
            return getRightExpression();
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }

    public void setExpression(int idx, Expression exp)
        throws ArrayIndexOutOfBoundsException{
        if (idx == 0)
            setLeftExpression(exp);
        else if (idx == 1)
            setRightExpression(exp);
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }


	// IMPLEMENTATION OF THE Expression INTERFACE //
	//--------------------------------------------//

	public PType getType() {
		return type;
	}

	public abstract void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException;

	// IMPLEMENTATION OF THE BinaryOperator INTERFACE //
	//------------------------------------------------//

	public void setLeftExpression(Expression leftChild)
		throws IllegalStateException {
		if (!verified) {
			left = leftChild;
		}
		else
			throw new IllegalStateException("Compiled expression can't be modified");
	}

	public void setRightExpression(Expression rightChild)
		throws IllegalStateException {
		if (!verified) {
			right = rightChild;
		}
		else
			throw new IllegalStateException("Compiled expression can't be modified");
	}

	public Expression getLeftExpression() {
		return left;
	}

	public Expression getRightExpression() {
		return right;
	}

	public boolean isCompiled() {
		return verified;
	}

	public Operand getResult() throws IllegalStateException {
		if (verified) {
			return result;
		}
		else
			throw new IllegalStateException("Can't get result of an uncompiled expression");
	}

	public Object clone() throws CloneNotSupportedException {
		BasicBinaryOperator bbo = (BasicBinaryOperator) super.clone();
		bbo.left = (Expression) bbo.left.clone();
		bbo.right = (Expression) bbo.right.clone();
		return bbo;
	}

}
