/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicFieldOperand
	extends BasicVariableOperand
	implements FieldOperand {

	Field field = null;
	int index = -1;

	public BasicFieldOperand(Field field) {
		super(field.getType());
		this.field = field;
	}

	public BasicFieldOperand(PType type, Field field) {
		super(type);
		this.field = field;
	}

	public PType getType() {
		return type;
	}

	public Field getField() {
		return field;
	}

	public void setField(Field f) {
		field = f;
		type = field.getType();
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int idx) {
		index = idx;
	}

	public String toString() {
		return "FieldOperand(" + (field == null ? "null" : field.getName() + ")");
	}

	public Object clone() throws CloneNotSupportedException {
		return super.clone();
		//BasicFieldOperand
	}

	public void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException {
		switch (type.getTypeCode()) {
		case QTypeTuple.TYPECODE_BOOLEAN:
			setValue(t.getBoolean(index));
			break;
		case QTypeTuple.TYPECODE_BYTE:
			setValue(t.getByte(index));
			break;
		case QTypeTuple.TYPECODE_CHAR:
			setValue(t.getChar(index));
			break;
		case QTypeTuple.TYPECODE_DATE:
			setValue(t.getDate(index));
			break;
		case QTypeTuple.TYPECODE_DOUBLE:
			setValue(t.getDouble(index));
			break;
		case QTypeTuple.TYPECODE_FLOAT:
			setValue(t.getFloat(index));
			break;
		case QTypeTuple.TYPECODE_INT:
			setValue(t.getInt(index));
			break;
		case QTypeTuple.TYPECODE_LONG:
			setValue(t.getLong(index));
			break;
		case QTypeTuple.TYPECODE_SHORT:
			setValue(t.getShort(index));
			break;
		case QTypeTuple.TYPECODE_STRING:
			setValue(t.getString(index));
			break;
		case QTypeTuple.TYPECODE_OBJBOOLEAN:
		case QTypeTuple.TYPECODE_OBJBYTE:
		case QTypeTuple.TYPECODE_OBJCHAR:
		case QTypeTuple.TYPECODE_OBJDOUBLE:
		case QTypeTuple.TYPECODE_OBJFLOAT:
		case QTypeTuple.TYPECODE_OBJINT:
		case QTypeTuple.TYPECODE_OBJLONG:
		case QTypeTuple.TYPECODE_OBJSHORT:
			setValue(t.getObject(index));
			break;
		}
		super.evaluate(pos, t);
	}
}
