/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.QType;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicParameterOperand
	extends BasicVariableOperand
	implements ParameterOperand {

	protected String name = null;

	public BasicParameterOperand(PType type, String name) {
		super(type);
		this.name = name;
	}

	public BasicParameterOperand(PType type, String name, Object value) {
		super(type);
		this.name = name;
		objectValue = value;
	}

	public BasicParameterOperand(PType type, String name, String value) {
		super(type);
		this.name = name;
		objectValue = value;
	}

	public BasicParameterOperand(PType type, String name, boolean value) {
		super(type);
		this.name = name;
		longValue = (value ? 1 : 0);
	}

	public BasicParameterOperand(PType type, String name, long value) {
		super(type);
		this.name = name;
		longValue = value;
	}

	public BasicParameterOperand(PType type, String name, double value) {
		super(type);
		this.name = name;
		doubleValue = value;
	}

	public String getName() {
		return name;
	}

	public void setName(String n) {
		name = n;
	}

	public void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException {
		int i = 0;
		while (i < pos.length
			&& pos[i].getType().getTypeCode() != type.getTypeCode()
			&& !pos[i].getName().equalsIgnoreCase(name)) {
			i++;
		}
		if (i < pos.length) {
			switch (type.getTypeCode()) {
			case QType.TYPECODE_BOOLEAN:
				setValue(pos[i].getBoolean());
				break;
			case QType.TYPECODE_BYTE:
				setValue(pos[i].getByte());
				break;
			case QType.TYPECODE_CHAR:
				setValue(pos[i].getChar());
				break;
			case QType.TYPECODE_DATE:
				setValue(pos[i].getDate());
				break;
			case QType.TYPECODE_DOUBLE:
				setValue(pos[i].getDouble());
				break;
			case QType.TYPECODE_FLOAT:
				setValue(pos[i].getFloat());
				break;
			case QType.TYPECODE_INT:
				setValue(pos[i].getInt());
				break;
			case QType.TYPECODE_LONG:
				setValue(pos[i].getLong());
				break;
			case QType.TYPECODE_SHORT:
				setValue(pos[i].getShort());
				break;
			case QType.TYPECODE_STRING:
				setValue(pos[i].getString());
				break;
			case QType.TYPECODE_NAMING_CONTEXT:
			case QTypeTuple.TYPECODE_OBJBOOLEAN:
			case QTypeTuple.TYPECODE_OBJBYTE:
			case QTypeTuple.TYPECODE_OBJCHAR:
			case QTypeTuple.TYPECODE_OBJDOUBLE:
			case QTypeTuple.TYPECODE_OBJFLOAT:
			case QTypeTuple.TYPECODE_OBJINT:
			case QTypeTuple.TYPECODE_OBJLONG:
			case QTypeTuple.TYPECODE_OBJSHORT:
			default:
				objectValue = pos[i].getObject();
				break;
			}
		}
		else
			throw new MedorException("Parameter without value: " + this.getName());
	}
}
