/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.ThreeArgsOperator;
import org.objectweb.medor.filter.api.VariableOperand;
import org.objectweb.medor.tuple.api.Tuple;

public abstract class BasicThreeArgsOperator implements ThreeArgsOperator {

    protected BasicThreeArgsOperator(Expression first, Expression second, Expression third) {
        this.first = first;
        this.second = second;
        this.third = third;
    }

    protected BasicThreeArgsOperator() {
    }

    protected PType type = null;
    protected Expression first, second = null, third = null;
    protected VariableOperand result = null;
    protected Operand firstResult = null, secondResult = null, thirdResult = null;
    protected boolean verified = false;

    // IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

    public int getOperandNumber() {
        return 3;
    }

    public Expression getExpression(int idx)
        throws ArrayIndexOutOfBoundsException {
        if (idx == 0)
            return getFirstExpression();
        else if (idx == 1)
            return getSecondExpression();
        else if (idx == 2)
            return getThirdExpression();
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }

    public void setExpression(int idx, Expression exp)
        throws ArrayIndexOutOfBoundsException{
        if (idx == 0)
            setFirstExpression(exp);
        else if (idx == 1)
            setSecondExpression(exp);
        else if (idx == 2)
            setThirdExpression(exp);
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }

    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public PType getType() {
        return type;
    }

    public abstract void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException;

    // IMPLEMENTATION OF THE BinaryOperator INTERFACE //
    //------------------------------------------------//

    public void setFirstExpression(Expression firstChild)
            throws IllegalStateException {
        if (!verified) {
            first = firstChild;
        } else
            throw new IllegalStateException("Compiled expression can't be modified");
    }

    public void setSecondExpression(Expression secondChild)
            throws IllegalStateException {
        if (!verified) {
            second = secondChild;
        } else
            throw new IllegalStateException("Compiled expression can't be modified");
    }

    public void setThirdExpression(Expression thirdChild)
            throws IllegalStateException {
        if (!verified) {
            third = thirdChild;
        } else
            throw new IllegalStateException("Compiled expression can't be modified");
    }

    public Expression getFirstExpression() {
        return first;
    }

    public Expression getSecondExpression() {
        return second;
    }

    public Expression getThirdExpression() {
        return third;
    }

    public boolean isCompiled() {
        return verified;
    }

    public Operand getResult() throws IllegalStateException {
        if (verified) {
            return result;
        } else
            throw new IllegalStateException("Can't get result of an uncompiled expression");
    }

    public Object clone() throws CloneNotSupportedException {
        BasicThreeArgsOperator bbo = (BasicThreeArgsOperator) super.clone();
        bbo.first = (Expression) bbo.first.clone();
        bbo.second = (Expression) bbo.second.clone();
        bbo.third = (Expression) bbo.third.clone();
        return bbo;
    }
}
