/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.UnaryArithmeticOperator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.QTypeTuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicUnaryArithmeticOperator
        extends BasicUnaryOperator
        implements UnaryArithmeticOperator {

    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//
    public PType getType() {
        return type;
    }

    public void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException, IllegalStateException {
        try {
            operand.evaluate(pos, t);
            switch (subResult.getType().getTypeCode()) {
                case QTypeTuple.TYPECODE_DOUBLE:
                    result.setValue(evaluate(subResult.getDouble()));
                    break;
                case QTypeTuple.TYPECODE_FLOAT:
                    result.setValue(evaluate(subResult.getFloat()));
                    break;
                case QTypeTuple.TYPECODE_BYTE:
                    result.setValue(evaluate(subResult.getByte()));
                    break;
                case QTypeTuple.TYPECODE_SHORT:
                    result.setValue(evaluate(subResult.getShort()));
                    break;
                case QTypeTuple.TYPECODE_INT:
                    result.setValue(evaluate(subResult.getInt()));
                    break;
                case QTypeTuple.TYPECODE_LONG:
                    result.setValue(evaluate(subResult.getLong()));
                    break;
                default:
                    throw new MalformedExpressionException(
                            "Unauthorized expression element");
            }
        }catch (NullPointerException e) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
    }

    public void compileExpression()
		throws TypingException, MalformedExpressionException {
        if (operand != null) {
            operand.compileExpression();
            if (PTypeSpaceMedor.isArithmeticType(operand.getType())) {
                type = operand.getType();
                subResult = operand.getResult();
                result = new BasicVariableOperand(type);
                verified = true;
            } else
                throw new TypingException("Attempt an aritmetic type");
        } else
            throw new MalformedExpressionException("null childeren value");// Operands not yet assigned
    }

    // IMPLEMENTATION OF THE UnaryArithmeticOperator INTERFACE //
    //---------------------------------------------------------//
    public abstract int evaluate(byte op) throws TypingException;

    public abstract int evaluate(short op) throws TypingException;

    public abstract int evaluate(int op) throws TypingException;

    public abstract long evaluate(long op) throws TypingException;

    public abstract float evaluate(float op) throws TypingException;

    public abstract double evaluate(double op) throws TypingException;
}
