/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.UnaryLogicalOperator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.QTypeTuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicUnaryLogicalOperator
        extends BasicUnaryOperator
        implements UnaryLogicalOperator {

    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException, IllegalStateException {
        try {
            operand.evaluate(pos, t);
            result.setValue(subResult.getBoolean());
        }catch (NullPointerException e) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
    }

    public void compileExpression()
		throws TypingException, MalformedExpressionException {
        if (operand != null) {
            operand.compileExpression();
            if (operand.getType().getTypeCode() ==
				QTypeTuple.TYPECODE_BOOLEAN) {
                type = PTypeSpaceMedor.BOOLEAN;
                result = new BasicVariableOperand(type);
                verified = true;
            } else
			    //it is not a boolean type
                throw new TypingException("Attempt a boolean type");
        } else
		    //Operands not yet assigned
            throw new MalformedExpressionException("Null children value");
    }

    // IMPLEMENTATION OF THE UnaryLogicalOperator INTERFACE //
    //-------------------------------------------------------//

    public abstract boolean evaluate(boolean boole);

}
