/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.UnaryOperator;
import org.objectweb.medor.filter.api.VariableOperand;
import org.objectweb.medor.tuple.api.Tuple;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicUnaryOperator implements UnaryOperator {

    protected PType type;
    Expression operand = null;
    protected VariableOperand result = null;
    protected Operand subResult = null;
    boolean verified = false;

    // IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

    public int getOperandNumber() {
        return 1;
    }

    public Expression getExpression(int idx)
        throws ArrayIndexOutOfBoundsException {
        if (idx == 0)
            return getExpression();
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }

    public void setExpression(int idx, Expression exp)
        throws ArrayIndexOutOfBoundsException{
        if (idx == 0)
            setExpression(exp);
        else
            throw new ArrayIndexOutOfBoundsException(idx);
    }

    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public PType getType(){
        return type;
    }

    public abstract void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException,IllegalStateException;

    // IMPLEMENTATION OF THE BinaryOperator INTERFACE //
    //------------------------------------------------//

    public void setExpression(Expression e) throws IllegalStateException {
        if (!verified){
            operand = e;
        }
		else
			throw new IllegalStateException("Compiled expression can't be modified");
    }

    public Expression getExpression() {
        return operand;
    }

    public Operand getResult() throws IllegalStateException {
        if (verified) {
            return result;
        }
		else
			throw new IllegalStateException("Can't get result of an uncompiled expression");
    }

    public boolean isCompiled(){
        return verified;
    }

    public Object clone() throws CloneNotSupportedException {
        BasicUnaryOperator buo = (BasicUnaryOperator) super.clone();
        buo.operand = (Expression) buo.operand.clone();
        return buo;
    }
}
