/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operator;

import java.util.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.Collator;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class Equal extends BasicComparator {

	private int strength = Collator.PRIMARY;

	public Equal() {
	}

	public Equal(int strength) {
		this.strength = strength;
	}

	public Equal(Expression l, Expression r) {
		this.left = l;
		this.right = r;
	}

	public boolean evaluate(Object op1, Object op2) {
		boolean result = false;
		if (op1 instanceof Character) {
			Character o1 = (Character) op1;
			Character o2 = (Character) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Long) {
			Long o1 = (Long) op1;
			Long o2 = (Long) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Short) {
			Short o1 = (Short) op1;
			Short o2 = (Short) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Integer) {
			Integer o1 = (Integer) op1;
			Integer o2 = (Integer) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Float) {
			Float o1 = (Float) op1;
			Float o2 = (Float) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Double) {
			Double o1 = (Double) op1;
			Double o2 = (Double) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Byte) {
			Byte o1 = (Byte) op1;
			Byte o2 = (Byte) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Date) {
			Date o1 = (Date) op1;
			Date o2 = (Date) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Time) {
			Time o1 = (Time) op1;
			Time o2 = (Time) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof Timestamp) {
			Timestamp o1 = (Timestamp) op1;
			Timestamp o2 = (Timestamp) op2;
			result = (o1.compareTo(o2) == 0);
		}
		else if (op1 instanceof String) {
			Collator myCollator = Collator.getInstance();
			myCollator.setStrength(strength);
			String o1 = (String) op1;
			String o2 = (String) op2;
			result = (myCollator.compare(o1, o2) == 0);
		}
		return result;
	}

	public boolean evaluate(int op1, int op2) {
		return (op1 == op2);
	}

	public boolean evaluate(int op1, short op2) {
		return (op1 == op2);
	}

	public boolean evaluate(int op1, long op2) {
		return (op1 == op2);
	}

	public boolean evaluate(int op1, float op2) {
		return (op1 == op2);
	}

	public boolean evaluate(int op1, double op2) {
		return (op1 == op2);
	}

	public boolean evaluate(float op1, float op2) {
		return (op1 == op2);
	}

	public boolean evaluate(float op1, short op2) {
		return (op1 == op2);
	}

	public boolean evaluate(float op1, int op2) {
		return (op1 == op2);
	}

	public boolean evaluate(float op1, long op2) {
		return (op1 == op2);
	}

	public boolean evaluate(float op1, double op2) {
		return (op1 == op2);
	}

	public boolean evaluate(char op1, char op2) {
		return (op1 == op2);
	}

	public boolean evaluate(long op1, short op2) {
		return (op1 == op2);
	}

	public boolean evaluate(long op1, int op2) {
		return (op1 == op2);
	}

	public boolean evaluate(long op1, long op2) {
		return (op1 == op2);
	}

	public boolean evaluate(long op1, float op2) {
		return (op1 == op2);
	}

	public boolean evaluate(long op1, double op2) {
		return (op1 == op2);
	}

	public boolean evaluate(double op1, short op2) {
		return (op1 == op2);
	}

	public boolean evaluate(double op1, int op2) {
		return (op1 == op2);
	}

	public boolean evaluate(double op1, long op2) {
		return (op1 == op2);
	}

	public boolean evaluate(double op1, float op2) {
		return (op1 == op2);
	}

	public boolean evaluate(double op1, double op2) {
		return (op1 == op2);
	}

	public boolean evaluate(boolean op1, boolean op2) {
		return (op1 == op2);
	}

	public boolean evaluate(String op1, String op2) {
		Collator myCollator = Collator.getInstance();
		myCollator.setStrength(strength);
		return (myCollator.compare(op1, op2) == 0);
	}

	public String getOperatorString() {
		return Operator.EQUAL;
	}
}
