/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.CalculatedParameterOperand;
import org.objectweb.medor.filter.api.BinaryOperator;
import org.objectweb.medor.filter.api.UnaryOperator;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.api.Field;
import org.objectweb.jorm.type.api.PType;

/**
 * @author S.Chassande-Barrioz
 */
public class ExpressionPrinter {
	public static String e2str(Expression e) {
		if (e instanceof FieldOperand) {
			Field f = ((FieldOperand) e).getField();
			if (f == null)
				return e.getType().getJavaName() + "nullFO";
			else
				return f.getName();
		}
		else if (e instanceof CompositePName) {
			FieldOperand[] fields = ((CompositePName) e).getFields();
			String result = "CompositePName(";
			Field f;
			for(int i=0; i<fields.length; i++) {
				f = fields[i].getField();
				if (f==null)
					result += "no field ,";
				else
					result += f.getName() + ", ";
			}
            result += ((CompositePName) e).getExpression(0);
			result += ")";
            return result;
		}
		else if(e instanceof SinglePName) {
            String res = "SinglePName(";
            Operand op = (Operand) ((SinglePName) e) .getRightExpression();
            Field f = ((SinglePName) e).getField().getField();
            res += (f == null ? null : f.getName()) + ", ";
            if (op instanceof ParameterOperand) {
                res += ((ParameterOperand) op).getName();
            } else {
                res += op.getObject();
            }
            res += ")";
            return res;
		}
		else if (e instanceof Operand) {
			PType type = e.getType();
			if (e instanceof ParameterOperand) {
				return
					(e instanceof CalculatedParameterOperand
						? "CalculatedParam("
						: "Param(")
					+ ((ParameterOperand) e).getName() + ", "
					+ (type == null ? "nullType" : type.getJormName()) + ","
					+ e.toString() + ")";
			}
			else
				return e.toString();
		} else if (e instanceof BinaryOperator) {
			BinaryOperator bo = (BinaryOperator) e;
			return "("
				+ e2str(bo.getLeftExpression())
				+ " " + bo.getOperatorString() + " "
				+ e2str(bo.getRightExpression())
				+ ")";
		} else if (e instanceof UnaryOperator) {
			UnaryOperator uo = (UnaryOperator) e;
			return uo.getOperatorString()
				+ e2str(uo.getExpression());
		} else if (e instanceof Operator) {
			Operator op = (Operator) e;
			StringBuffer s = new StringBuffer();
			s.append(op.getOperatorString());
            s.append("(");
            String sep = "";
			for (int i=0; i< op.getOperandNumber(); i++) {
				s.append(sep);
                sep = ", ";
				s.append(e2str(op.getExpression(i)));
			}
			s.append(")");
			return s.toString();
		}
		return "";
	}

}
