/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;

import java.util.Collection;
import java.util.Iterator;

/**
 * The InCollection class is an operator testing whether the value of a
 * Field (the left operand, which is supposed to be a FieldOperand) is equal to
 * one of the elements of a collecton (the right Operand)
 */
public class InCollection extends BasicBinaryOperator {

	/**
	 * elemType is the type of the elements of the collection
	 */
	private PType elemType;

	/**
	 * Returns the declared PType of the elements.
	 */
	public PType getElemType() {
		return elemType;
	}


	public InCollection(FieldOperand left, Operand right,
						PType elemType) {
		this.left = left;
		this.right = right;
		this.elemType = elemType;
	}

	public PType getType() {
		 return PTypeSpaceMedor.BOOLEAN;
	}

	public void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException, IllegalStateException {
		try {
			left.evaluate(pos, t);
			Collection c = null;
			if (right instanceof ParameterOperand) {
				String pn = ((ParameterOperand) right).getName();
				if (pos == null || pos.length == 0)
					throw new MedorException("No Parameter");
				int i = 0;
				while (i < pos.length && !pn.equals(pos[i].getName()))
					i++;
				if (i < pos.length)
					c = (Collection) pos[i].getObject();
				else
					throw new MedorException("Parameter " + pn + " not found");
			} else
				c = (Collection) ((Operand) right).getObject();
			Iterator it = c.iterator();
			boolean in = false;
			Object o = ((FieldOperand)left).getObject();
			while (it.hasNext() && !in)
				in = it.next().equals(o);
			result.setValue(in);
		}
		catch (NullPointerException e) {
			// result == null or missing children objects
			throw new IllegalStateException("Unevaluable Expression: Not compiled");
		}
	}

	public void compileExpression()
		throws TypingException, MalformedExpressionException {
		if ((left != null) && (right != null)) {
			result = new BasicVariableOperand(type);
			verified = true;
		}
		else
		// Operands not yet assigned
			throw new MalformedExpressionException("null children value");
	}

	public String getOperatorString() {
		return Operator.IN;
	}
}
