/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.UnaryOperator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 * IsEmpty operates on an Expression which must be of a TupleCollection type,
 * or a GenClassRef. Its evaluation returns true if the Expression is an empty
 * TupleCollection or if the GenClassRef has no element.
 */
public class IsEmpty extends BasicUnaryOperator implements UnaryOperator {

	public IsEmpty() {
	}

	public IsEmpty(Expression e) {
		this.operand = e;
	}

	public void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException, IllegalStateException {
		try {
			operand.evaluate(pos, t);
			switch (subResult.getType().getTypeCode()) {
			case QTypeTuple.TYPECODE_REFERENCE:
				//TODO: raffiner pour les genclassref
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_TUPLE_COLLECTION:
				result.setValue(evaluate(subResult.getObject()));
				break;
			default:
				throw new MalformedExpressionException(
					"Unauthorized expression element");
			}
		}
		catch (NullPointerException e) {
			throw new IllegalStateException("Unevaluable Expression: Not compiled");
		}
	}

	public boolean evaluate(Object op) {
		//TODO
		return true;
	}

	public void compileExpression()
		throws TypingException, MalformedExpressionException {
		if (operand != null) {
			operand.compileExpression();
			//TODO raffiner pour les genclassref
			if ((operand.getType().getTypeCode()==QTypeTuple.TYPECODE_REFERENCE) ||
				(operand.getType().getTypeCode()==QTypeTuple.TYPECODE_TUPLE_COLLECTION)) {
				type = PTypeSpaceMedor.BOOLEAN;
				subResult = operand.getResult();
				result = new BasicVariableOperand(type);
				verified = true;
			}
			else {
				throw new TypingException("IsEmpty only operates on GenClassReferences and TupleCollections");
			}
		}
		else
		//Operands not yet assigned
			throw new MalformedExpressionException("null children value");
	}

	public String getOperatorString() {
		//TODO
		return "TODO";
	}
}
