/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.UnaryOperator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 * IsNull operates on an Expression. Its evaluation returns true if the
 * Expression is null, false otherwise.
 */
public class IsNull extends BasicUnaryOperator implements UnaryOperator {

	boolean not = false;

    public IsNull() {
	}

	public IsNull(Expression e) {
		this.operand = e;
	}

	public IsNull(Expression e, boolean not) {
		this.operand = e;
        this.not = not;
	}

    public void setNot(boolean not) {
        this.not = not;
    }

    public boolean getNot() {
        return not;
    }

	public void evaluate(ParameterOperand[] pos, Tuple t)
		throws MedorException, IllegalStateException {
		try {
			operand.evaluate(pos, t);
			switch (subResult.getType().getTypeCode()) {
			case QTypeTuple.TYPECODE_OBJBOOLEAN:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJCHAR:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJBYTE:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJSHORT:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJINT:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJLONG:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJFLOAT:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_OBJDOUBLE:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_STRING:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_DATE:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_CHARARRAY:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_BYTEARRAY:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_SERIALIZED:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_REFERENCE:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_TUPLE:
				result.setValue(evaluate(subResult.getObject()));
				break;
			case QTypeTuple.TYPECODE_TUPLE_COLLECTION:
				result.setValue(evaluate(subResult.getObject()));
				break;
			default:
				throw new MalformedExpressionException(
					"Unauthorized expression element");
			}
		}
		catch (NullPointerException e) {
			throw new IllegalStateException("Unevaluable Expression: Not compiled");
		}
	}

	public boolean evaluate(Object op) {
		return (not ? op != null : op == null);
	}

	public void compileExpression()
		throws TypingException, MalformedExpressionException {
		if (operand != null) {
			operand.compileExpression();
			if ((PTypeSpaceMedor.isArithmeticType(operand.getType())) ||
				(operand.getType().getTypeCode()==QTypeTuple.TYPECODE_BOOLEAN) ||
				(operand.getType().getTypeCode()==QTypeTuple.TYPECODE_CHAR) ||
				(operand.getType().getTypeCode()==QTypeTuple.TYPECODE_BYTE)) {
				throw new TypingException("IsNull does not work on primitive types");
			}
			else {
				type = PTypeSpaceMedor.BOOLEAN;
				subResult = operand.getResult();
				result = new BasicVariableOperand(type);
				verified = true;
			}
		}
		else
		//Operands not yet assigned
			throw new MalformedExpressionException("null children value");
	}

	public String getOperatorString() {
		return (not ? "IS NOT NULL" : "IS NULL");
	}
}
