/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.tuple.api.Tuple;

/**
 * @author S.Chassande-Barrioz
 */
public class Like implements Operator {

	public String getOperatorString() {
		return (not ? "NOT LIKE" : "LIKE");
	}

	Expression[] param;
	boolean not = false;

	public Like(Expression str, Expression pattern, Expression escape) {
		param = new Expression[] {str, pattern, escape};
	}

	public Like(Expression str, Expression pattern, Expression escape, boolean not) {
		param = new Expression[] {str, pattern, escape};
		this.not = not;
	}

	public Like(Expression str, Expression pattern) {
		param = new Expression[] {str, pattern};
	}

	public Like(Expression str, Expression pattern, boolean not) {
		param = new Expression[] {str, pattern};
		this.not = not;
	}

	// IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

	public PType getType() {
		return PTypeSpace.BOOLEAN;
	}

	public Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException();
	}

	public int getOperandNumber() {
		return param.length;
	}

	public boolean isCompiled() {
		return false;
	}

	public Expression getExpression(int idx) throws ArrayIndexOutOfBoundsException {
		return param[idx];
	}

	public void compileExpression()
		throws TypingException, MalformedExpressionException {
	}

	public void setExpression(int idx, Expression exp) throws ArrayIndexOutOfBoundsException {
		param[idx] = exp;
	}

	public Operand getResult() throws IllegalStateException {
		return null;
	}

	public void evaluate(ParameterOperand[] values, Tuple t)
		throws MedorException, IllegalStateException {
		throw new MedorException("Not yet implemented");
	}
}
