/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.ThreeArgsOperator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 * Operator representing the substring extraction given a String, a first
 * integer to start from and a second integer indicating the length of the
 * required substring.
 */
public class Substring extends BasicThreeArgsOperator
	implements ThreeArgsOperator {

	/**
	 * Constructs the operator
	 * @param inputString is the String from which to extract the substing
	 * @param start is the index of the first character in the String to start
	 * with. It is expected that 1 is the first character in the string (and
	 * not 0 like in Java).
	 * @param length is the size of the requested substring
	 */
    public Substring(Expression inputString, Expression start,
					 Expression length) {
        super(inputString, start, length);
    }

    public Substring() {
    }

    public void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException, IllegalStateException {
        try {
            first.evaluate(pos, t);
            second.evaluate(pos, t);
            third.evaluate(pos, t);
            result.setValue(evaluate(firstResult.getString(),
                    secondResult.getInt(), thirdResult.getInt()));
        } catch (NullPointerException e) {
            // result == null or missing children objects
            throw new IllegalStateException("Unevaluable Expression: May be Not compiled");
        }
    }

    public void compileExpression()
            throws TypingException, MalformedExpressionException {
        if ((first != null) && (second != null) && (third != null)) {
            first.compileExpression();
            second.compileExpression();
            third.compileExpression();
            if ((first.getType().getTypeCode() ==
                    QTypeTuple.TYPECODE_STRING) &&
                    (second.getType().isa(PTypeSpaceMedor.INT)) &&
                    (third.getType().isa(PTypeSpaceMedor.INT))) {
                firstResult = first.getResult();
                secondResult = second.getResult();
                thirdResult = third.getResult();
                type = PTypeSpaceMedor.STRING;
                result = new BasicVariableOperand(type);
                verified = true;
            } else
            //it is not a boolean type
                throw new TypingException("Attempt Arg Types: String, int , int");
        } else
        //Operands not yet assigned
            throw new MalformedExpressionException("Null children value");
    }

    public String evaluate(String op, int start, int length) {
        return op.substring(start-1, length + start-1);
    }

    public String getOperatorString() {
        return Operator.SUBSTRING;
    }
}
