/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.optim.api;

import java.util.List;

/**
 * This interface describes the configuration of a set of rules used by
 * the QueryRewriter.
 */
public interface RuleConfiguration {
	/**
	 * Adds a rule to a configuration, together with parameters such
	 * as the number of times the rules is applied (once, saturation)
	 * @param rule the rule to be added.
	 */
	void addRule(RewriteRule rule);

	/**
	 * Deletes a rule from the configuration.
	 * @param rule the rule to be removed
	 */
	void removeRule(RewriteRule rule);

	/**
	 * Gives the List of rules in the configuration
	 * @return the List of rules in the configuration
	 */ List getRules();
}
