/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.optim.jorm;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.rdb.api.RdbClassMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbPrimitiveElementMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbGenClassMapping;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.jorm.api.JormExtent;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.jorm.lib.GenClassExtent;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author S.Chassande-Barrioz
 * @deprecated use FlattenQueryTreeRule and JormFlatten2Rdb
 * @see org.objectweb.medor.optim.lib.FlattenQueryTreeRule
 * @see JormFlatten2Rdb
 */
public class RdbJormLeafRewriter extends JormLeafRewriter {

	public final static String MAPPER_NAME = "rdb";

	public RdbJormLeafRewriter() {
		super("RdbJormLeafRewriter");
	}

	public boolean canRewrite(QueryLeaf ql) {
        try {
            if (ql instanceof ClassExtent) {
                getClassMapping((ClassExtent) ql);
            } else if (ql instanceof GenClassExtent) {
                getGenClassMapping((GenClassExtent) ql);
            } else {
                return false;
            }
            return true;
        } catch (MedorException e) {
            return false;
        }
	}

	public QueryLeaf createQueryLeaf(JormExtent extent) throws MedorException {
        String tablename;
        if (extent instanceof ClassExtent) {
            tablename = ((RdbClassMapping) getClassMapping(extent))
                .getRdbTable().getName();
        } else if (extent instanceof GenClassExtent) {
            tablename = ((RdbGenClassMapping) getGenClassMapping(extent))
                .getRdbTable().getName();
        } else {
            throw new MedorException("Unmanaged JormExtent: " + extent);
        }
		BasicQualifiedTable qt = new BasicQualifiedTable(tablename, null);
		DataStore ds = extent.getDataStore();
		if (ds == null) {
			Object cf = extent.getPMapper().getConnectionFactory();
			ds = new ConnectionFactoryDataStore(
				DataStore.JDBC_STORE,
				cf.toString(),
				new short[0],
				cf);
		}
		RdbExpQueryLeaf neo = new BasicRdbExpQueryLeaf(
			ds,
			new BasicQualifiedTable[]{qt},
			extent.getName());
		neo.setRdbAdapterName(extent.getPMapper().getMapperName());
		return neo;
	}

	public QueryTreeField addPrimitiveElement(QueryLeaf neo,
											  PrimitiveElement pe,
											  JormExtent extent)
		throws MedorException {
		return ((RdbExpQueryLeaf) neo).addRdbField(
			getFieldName(neo.getName(), pe.getName()),
			pe.getType(),
			getColumnName(pe, extent),
			((RdbExpQueryLeaf) neo).getQualifiedTables()[0]);
	}

	private String getColumnName(PrimitiveElement pe,
								 JormExtent extent) throws MedorException {
		return ((RdbPrimitiveElementMapping) getPEM(extent, pe.getName())).getName();
	}
}
