/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.optim.lib;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.lib.Log;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.optim.api.RewriteRule;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.api.MedorException;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class BasicRule implements RewriteRule {

	/**
	 * A ModifiedExpression is a structure containing an Expression and a
	 * boolean indicating whether the Expression has been modified or not.
	 * <p>
	 * It is used by RewriteRules to check whether an Expression has been
	 * modified, in order to propagate the changes.
	 */
	public class ModifiedExpression {
		public boolean isModified;
		public Expression e;

		public ModifiedExpression() {
        }

        public ModifiedExpression(Expression ex, boolean ism) {
            e = ex;
            isModified = ism;
        }
	}

	protected Logger log = null;

	protected boolean debug = false;

	public BasicRule() {
		log = Log.loggerFactory.getLogger(
			Log.MEDOR_PREFIX + ".optim.rule");
		debug = log != null && log.isLoggable(BasicLevel.DEBUG);
	}

	public BasicRule(String suffix) {
		log = Log.loggerFactory.getLogger(
			Log.MEDOR_PREFIX + ".optim.rule." + suffix);
		debug = log != null && log.isLoggable(BasicLevel.DEBUG);
	}

	public QueryTree rewrite(QueryTree qt) throws MedorException {
		return null;
	}
}
