/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.optim.rdb;

import org.objectweb.jorm.runtime.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.optim.lib.BasicRule;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Map;

/**
 * @author S.Chassande-Barrioz
 */
public class RdbAssignRdbAdapterRule extends BasicRule {

    protected Map name2Adapter = null;

    public RdbAssignRdbAdapterRule() {
        super("RdbAssignRdbAdapterRule");
    }

    public RdbAssignRdbAdapterRule(Map m) {
        this();
        name2Adapter = m;
    }

    public void setName2Adapter(Map m) {
        name2Adapter = m;
    }

    public Map getName2Adapter() {
        return name2Adapter;
    }

    public QueryTree rewrite(QueryTree qt) throws MedorException {
        log.log(BasicLevel.DEBUG, "Assignation of relational adapters");
        if (name2Adapter != null && name2Adapter.size()>0)
            assignAdapter(qt);
        return qt;
    }

    protected void assignAdapter(QueryTree qt) throws MedorException {
        if (qt instanceof RdbExpQueryLeaf) {
            RdbExpQueryLeaf ql = (RdbExpQueryLeaf) qt;
            String name = ql.getRdbAdapterName();
            if (name == null) {
                log.log(BasicLevel.DEBUG,
                    "No adapter specified for the RdbExpQueryLeaf: " + ql.getName());
                return;
            }
            RdbAdapter adapter = (RdbAdapter)
                name2Adapter.get(name);
            if (adapter != null) {
                ql.setRdbAdapter(adapter);
                log.log(BasicLevel.DEBUG, "Assign the adapter "
                    + ql.getRdbAdapterName() + "to the RdbExpQueryLeaf: "
                    + ql.getName());
            } else
                throw new MedorException("No adapter " + name
                    + " found for the RdbExpQueryLeaf: " + ql.getName());
        } else if (qt instanceof QueryNode) {
            QueryTree[] qts = ((QueryNode) qt).getChildren();
            for (int i = 0; i < qts.length; i++) {
                assignAdapter(qts[i]);
            }
        }
    }

}
