/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query.api;

import org.objectweb.medor.filter.api.Expression;

/**
 * This interface is implemented by all QueryTree which have a QueryFilter
 * associated (QueryNode, RdbExpLeaf, ...).
 *
 * @author S.Chassande-Barrioz
 */
public interface FilteredQueryTree extends QueryTree {

	/**
	 * Retrieves the filter associated to the QueryTree. This filter is
	 * represented by an Expression.
	 * @return the query filter Expression associated to the QueryNode.
	 * @see Expression
	 */
	Expression getQueryFilter();

	/**
	 * Sets the filter associated to the FilteredQueryTree. This filter is
	 * represented by an Expression.
	 * <p>
	 * For a QueryNode:
	 * <p>
	 * This method also updates the list of QueryTree children of the current
	 * QueryNode according to the FieldOperands present in the Expression.
	 * <p>
	 * <b>Usage constraint:</b> if the QueryNode already has an associated
	 * Expression, and if this Expression has been modified, the user is
	 * requested to call the setQueryFilter method after the modification.
	 * <p>
	 * For a RbdQueryLeaf:
	 * <p>
	 * It sets the filter associated to the RbdQueryLeaf. This filter is
	 * represented by an Expression. Unlike the filter of a QueryNode, is
	 * applies to its own Fields, instead of the Fields of its children.
	 * @param e is the Expression representing the query filter of the current
	 * QueryNode.
	 * @throws UnsupportedOperationException when it is not supported
	 * (for Union, Intersection, Cartesian, Projection).
	 * @see Expression
	 */
	void setQueryFilter(Expression e);
}
