/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.api;

/**
 * This interface is used for QueryTrees which can be optimized.
 */
public interface OptimizationMetaData {

	/**
	 * Set a boolean for declaring the use of a cache for intermediate
	 * materialization of results at evaluation time.
	 */
	public void setCache(boolean isCache);

	/**
	 * Get the value of the boolean indicating whether cache is used for
	 * materialization of intermediate results at evaluation time.
	 * @return the boolean indicating whether the cache is used for
	 * materialization of intermediate results.
	 */
	public boolean isCache();

	/**
	 * Set a boolean for declaring parallel evaluation of sub-QueryTrees.
	 */
	public void setParallel(boolean isTupleLoader);

	/**
	 * Get the value of the boolean indicating whether parallel evaluation
	 * of sub-QueryTrees is performed.
	 * @return the boolean indicating parallel evaluation of sub-QueryTrees is
	 * performed.
	 */
	public boolean isParallel();
}
