/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.api;

import org.objectweb.medor.api.Field;

/**
 * A PropagatedField is a Field that takes its value from Fields of the
 * children query trees (called previous fields).
 * @author Sebastien Chassande-Barrioz
 */
public interface PropagatedField extends QueryTreeField {

	Field[] getPreviousFields();

	Field[] getPreviousFields(String oldFieldName);

	Field[] getOriginFields();

	/**
	 * The index of this Field in the array made of the concatenation of the
	 * tuples from the children QueryTrees.
	 * This information is used to pick the right value at evaluation time
	 * from this concatenated tuple to put it in the result Tuple.
	 */
	int getIndex();

	void setIndex(int idx);

	void replacePreviousField(Field[] neo);
}
