/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.query.jorm.lib;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.jorm.api.JormExtent;
import org.objectweb.medor.query.jorm.api.JormField;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * This class is the common implementation of the jorm leaves (ClassExtent,
 * GenClassExtent).
 *
 * @author S.Chassande-Barrioz
 */
public abstract class BasicJormExtent
		implements JormExtent, QueryLeaf, TupleStructure {
	/**
	 * Node name
	 */
	protected String name;

	/**
	 * Name of the field which is the PName of the extent
	 */
	protected String pnFieldName;

	protected PNameField identifier = null;

	/**
	 * List of fields
	 */
	protected ArrayList fields = new ArrayList();

	/**
	 * List of fields indexed by their name
	 */
	protected HashMap name2field = new HashMap();

	/**
	 * The datastore of the extent
	 */
	protected DataStore myStore;

	/**
	 * The mapper of the extent.
	 */
	protected PMapper mapper;

	protected String projectName = null;

	public BasicJormExtent(String _name, String pnFieldName) {
		this.name = (_name == null ? "" : _name);
		;
		this.pnFieldName = pnFieldName;
	}

	public Iterator iterateFields() {
		return fields.iterator();
	}

	public String getFieldName(String nodeName, String fieldName) {
		if (nodeName == null || nodeName.length() == 0)
			return fieldName;
		else
			return nodeName + "." + fieldName;
	}

	// IMPLEMENTATION OF THE JormExtent INTERFACE //
	//--------------------------------------------//

	public PMapper getPMapper() {
		return mapper;
	}

	public void setPMapper(PMapper m, String projectName) {
		mapper = m;
		/*
		if (projectName == null && m != null) {
			projectName = m.getMappingName();
		}
		*/
		this.projectName = projectName;
	}

	public String getProjectName() {
		return projectName;
	}

	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}

	public void setDataStore(DataStore ds) {
		myStore = ds;
	}

	public String getPNameFieldName() {
		return getFieldName(name, pnFieldName);
	}

	public abstract String getJormName();

	public JormField getIdentifierField() {
		return identifier;
	}

	// IMPLEMENTATION OF THE QueryLeaf INTERFACE //
	//-------------------------------------------//

	public String getName() {
		return this.name;
	}

	/**
	 * Methods inherited from QueryTree
	 */
	public TupleStructure getTupleStructure() {
		return this;
	}

	/**
	 * Methods inherited from QueryLeaf
	 */
	public DataStore getDataStore() {
		return myStore;
	}

	public Object clone() {
//TODO
		return this;
	}

	// IMPLEMENTATION OF THE TupleStructure INTERFACE //
	//------------------------------------------------//

	public Field[] getFields() {
		return (Field[]) fields.toArray(new Field[0]);
	}

	public Field getField(String fieldname) throws MedorException {
		Field res = (Field) name2field.get(fieldname);
		if (res == null) {
			throw new MedorException("Field name error: " + fieldname);
		}
		return res;
	}

	public Field getField(int fieldrank) throws MedorException {
		Field f = (Field) fields.get(fieldrank - 1);
		if (f == null) {
			throw new MedorException("Field rank error : " + fieldrank);
		}
		else {
			return f;
		}
	}

	public int getFieldRank(Field f) throws MedorException {
		int idx = fields.indexOf(f);
		if (idx == -1)
			throw new MedorException("Field not found: " + f.getName());
		else
			return idx + 1;
	}

	public int getSize() {
		return fields.size();
	}

	public boolean contains(Field f) {
		return fields.contains(f);
	}

	public boolean contains(String fieldName) {
		return name2field.containsKey(fieldName);
	}

	public void setDistinct(boolean d) throws MedorException {
		throw new MedorException("Cannot set distinct on a JORM Leaf");
	}

	public boolean getDistinct() {
		return false;
	}

	public void setOrderBy(OrderField[] orderfields) throws MedorException {
		throw new MedorException("Order by not supported on MedorTCQueryLeaf");
	}

	public OrderField[] getOrderBy() {
		return null;
	}

}
