/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.lib;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.lib.BasicMultivaluedField;
import org.objectweb.medor.query.api.NestedField;
import org.objectweb.medor.query.api.QueryTree;

/**
 *  This class represents a TupleCollection Field. Notes that all fields blong
 * to the same QueryTree. It is used by the NestQueryNode and the UnnestQueryNode.
 */
public class BasicNestedField extends BasicMultivaluedField
        implements NestedField {

    QueryTree qt;

    public BasicNestedField(String name, QueryTree qt,
                            Field[] nestedFields) throws MedorException {
        super(name, nestedFields);
        this.qt = qt;
    }

    public QueryTree getQueryTree() {
        return qt;
    }

    public void replacePreviousField(Field[] neo) {
        name2field.clear();
        for (int i = 0; i < neo.length; i++) {
            name2field.put(neo[i].getName(), neo[i]);
        }
    }
}
