/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.lib;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.OptimizationMetaData;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class Cartesian extends BasicQueryNode {

	/**
	 *
	 * @param leftQT is the left QueryTree child used in the cartesian product.
	 * @param rightQT is the rigth QueryTree child used in the cartesian product.
	 */
	public Cartesian(QueryTree leftQT,
					 QueryTree rightQT,
                     String name,
					 OptimizationMetaData optimizationMD) {
		super(name, optimizationMD);
		//adds the Fields from the left and right QueryTrees
		Field[] leftFields = leftQT.getTupleStructure().getFields();
		Field[] rightFields = rightQT.getTupleStructure().getFields();
		try {

			for (int i = 0; i < leftFields.length; i++) {
				this.addPropagatedField(
					leftFields[i].getName(),
					leftFields[i].getType(),
					new QueryTreeField[]{(QueryTreeField) leftFields[i]});
			}
			for (int i = 0; i < rightFields.length; i++) {
				this.addPropagatedField(
					rightFields[i].getName(),
					rightFields[i].getType(),
					new QueryTreeField[]{(QueryTreeField) rightFields[i]});
			}
		}
		catch (MedorException e) {
		}
		//computes the indexes
		int lsize = leftQT.getTupleStructure().getSize();
		int rsize = rightQT.getTupleStructure().getSize();
		indexes = new int[lsize + rsize];

		Field[] old = null;
		for (int i = 1; (i <= lsize); i++) {
			indexes[i - 1] = i;
		}
		for (int i = lsize + 1; (i <= (lsize + rsize)); i++) {
			indexes[i - 1] = i;
		}
	}

	// IMPLEMENTATION OF THE QueryNode INTERFACE //
	//-------------------------------------------//

	public void setQueryFilter(Expression f) {
		throw new UnsupportedOperationException("Cartesian products cannot be assigned a filter.");
	}

	public short getType() {
		return OperationType.CARTESIAN;
	}

	// OTHER METHODS //
	//---------------//

	public Object clone() throws CloneNotSupportedException {
		Cartesian copy = (Cartesian) super.clone();
// TODO
/*
			copy.child[0] = (QueryTree) ((Object) child[0]).clone();
			copy.rightQT = (QueryTree) copy.rightQT.clone();
			copy.indexs = (int[]) copy.indexs.clone();
			AttributeChangeMetaData[] newInfo = new AttributeChangeMetaData[projectedAttributes.getAttributeProjectCount()];
			for (int i = 1; (i <= newInfo.length); i++) {
				String nName = (String) projectedAttributes.getAttributeNewName(i);
				String[] pName = (String[]) projectedAttributes.getAttributePastName(nName).clone();
				QueryTree[] qtf = new QueryTree[projectedAttributes.getQueryTreeFromNewPosition(i).length];
				for (int cpt = 0; (cpt < qtf.length); cpt++) {
					if (qtf[cpt] == leftQT)
						qtf[cpt] = copy.leftQT;
					else
						qtf[cpt] = copy.rightQT;
				}
				newInfo[i - 1] = new AttributeChangeMetaData(pName, qtf, nName, projectedAttributes.getAttributeType(i));
			}
			copy.projectedAttributes = new ProjectionMetaData(newInfo);
*/
		return copy;
	}

}
