/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query.lib;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.OptimizationMetaData;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;

public class Project extends BasicQueryNode {

	/**
	 * Basic constructor. The Fields must be added later.
	 */
	public Project(String nodeName, OptimizationMetaData optimizationMD) throws MedorException {
		super(nodeName, optimizationMD);
	}

	/**
	 * Constructs a Project node from an array of fieldNames to be projected
	 * from a given (child) QueryTree.
	 * @param fieldNames is the array of field names for which the Field is to
	 * be projected.
	 * @param child is the child QueryTree on which the projection operates.
	 */
	public Project(String[] fieldNames, QueryTree child, String nodeName,
				   OptimizationMetaData optimizationMD) throws MedorException {
		super(nodeName, optimizationMD);
		for (int i = 0; i < fieldNames.length; i++) {
			Field f = child.getTupleStructure().getField(fieldNames[i]);
			QueryTreeField[] anc = new QueryTreeField[1];
			anc[0] = (QueryTreeField) f;
			addPropagatedField(f.getName(), f.getType(), anc);
		}
	}

	public short getType() {
		return OperationType.PROJECT;
	}

	public void setQueryFilter(Expression e) {
		throw new UnsupportedOperationException("Projections cannot be assigned a filter.");
	}
}
