/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

/**
 * Package definition.
 */
package org.objectweb.medor.query.lib;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.query.api.NestedField;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.OptimizationMetaData;
import org.objectweb.medor.query.api.PropagatedField;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.UnnestQueryNode;
import org.objectweb.medor.type.lib.QType;


public class Unnest extends BasicQueryNode implements UnnestQueryNode {
    /**
     * It lists the field which have been grouped.
     */
    protected NestedField unnestedField = null;
    protected PropagatedField[] projectedFields;

    public Unnest(QueryTreeField[] queryTreeFields,
                  Field field,
                  String nodeName,
                  OptimizationMetaData optimizationMD) throws MedorException{

        super(nodeName, optimizationMD);

        //creating and adding all projected fields as PropagatedFields
		this.projectedFields = new PropagatedField[queryTreeFields.length];

		for (int i = 0; i < queryTreeFields.length; i++) {
			QueryTreeField[] anc = {queryTreeFields[i]};
			this.projectedFields[i] = this.addPropagatedField(
				queryTreeFields[i].getName(),
				queryTreeFields[i].getType(),
				anc);
		}


        if(field.getType().getTypeCode()!=QType.TYPECODE_TUPLE_COLLECTION)
            throw new TypingException("The field to unnest must be a TupleCollection");
        Field[] origs;
        if (field instanceof PropagatedField) {
            origs = ((PropagatedField) field).getOriginFields();
        }
        else
            origs = new Field[] { field };
        if (!(origs[0] instanceof  NestedField))
            throw new MedorException("The support of Tuplecollection fields (which are not NestedFields) is not yet implemented");

		unnestedField = (NestedField) origs[0];

		//creating and adding all fields coming from the unnest operation
		//as BasicPropagFromNestedField
		for (int i = 0; i < unnestedField.getSize(); i++) {
			QueryTreeField[] anc = {unnestedField};
            Field f = unnestedField.getFields()[i];
            addField(new BasicPropagFromNestedField(
				getFieldName(nodeName, f.getName()),
				f.getType(),
                this,
				anc,
                new Field[] {f}));
		}
        addChild(unnestedField.getQueryTree());
    }

    public NestedField getUnnestedField() {
        return unnestedField;
    }

    public PropagatedField[] getProjectedFields() {
        return projectedFields;
    }

    public short getType() {
        return OperationType.UNNEST;
    }

    public void setQueryFilter(Expression e) {
        throw new UnsupportedOperationException("Unnest operators cannot be assigned a filter.");
	}

}
