/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.api;

import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.ParameterOperand;

/**
 * An RdbQueryLeaf represents a query leaf which encapsulates access to a
 * relational database.
 * <p>
 * A SQL query is associated to an RdbQueryLeaf.
 * @see org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf
 * @see org.objectweb.medor.query.rdb.api.RdbStringQueryLeaf
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface RdbQueryLeaf extends QueryLeaf {

	/**
	 * Returns the associated SQL query.
	 * In the case of an RdbExpQueryLeaf, the query is computed.
	 * @return the SQL query associated to the RdbQueryLeaf, as a String.
	 * @throws MedorException in the case of RdbExpQueryLeaves if the
	 * SQL request cannot be computed from the associated Expression
	 */
	String getSqlRequest(ParameterOperand[] pos) throws MedorException;

}
