/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.rdb.api.RdbField;
import org.objectweb.medor.query.rdb.api.RdbStringQueryLeaf;
import org.objectweb.medor.query.lib.BasicQueryTree;

/**
 *
 * This class represents a QueryLeaf that maps onto a relational database store.
 */
public class BasicRdbStringQueryLeaf extends BasicQueryTree
	implements RdbStringQueryLeaf, TupleStructure {

	private boolean distinct = false;
	Expression sqlFilter;
	String request;
	DataStore ds;
    String nodeName;

	/**
	 * Constructs a BasicRdbStringQueryLeaf from an SQL string.
	 */
	public BasicRdbStringQueryLeaf(DataStore ds,
							 String request, String nodeName) {
		super(nodeName, null);
        this.nodeName = nodeName;
		this.ds = ds;
		this.request = request;
	}

	public RdbField addRdbField(String name,
								PType type,
								String colName) {
		RdbField f = new BasicRdbField(name, type, colName, this);
		name2field.put(f.getName(), f);
		fields.add(f);
		return f;
	}

	public String getSqlRequest(ParameterOperand[] pos) {
		return request;
	}

    public String getName() {
        return nodeName;
    }

	public boolean getDistinct() {
		return distinct;
	}

	public void setDistinct(boolean d) {
		distinct = d;
	}
    public DataStore getDataStore() {
		return ds;
	}

	public TupleStructure getTupleStructure() {
		return (TupleStructure) this;
	}
}
