/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.tuple.api;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.filter.api.VariableOperand;
import org.objectweb.medor.filter.api.ParameterOperand;

/**
 * This interface
 */
public interface TupleLoader {

    /**
     * Loads a source Tuple into an array of Operands.
     * The Tuple represents the concatenation of the Tuples from the children
     * nodes of the associated QueryNode.
     * LoadTuple is used after the filter has been sucessfully evaluated to
     * load the current tuple (Tuple) into the memory tuple of this node,
     * represented as an array of Operands.
     * Not all attributes of the source Tuple are loaded into the array of
     * Operand. The choice of which attributes to load is done either
     * by using the array of indexes, which can be set through the setIndexes
     * method, or because the instance of TupleLoader has been compiled for
     * a particular QueryNode for which the indexes are known, and the
     * implementation of loadTuple does the right selection amongst the
     * Tuple attributes.
     * @param source The source tuple to be loaded
     * @param destination The array of VariableOperands into which the Tuple
	 * will be loaded
	 * @param parameters The array of input ParameterOperands.
     */
    public void loadTuple(Tuple source, VariableOperand[] destination,
						  ParameterOperand[] parameters)
            throws MedorException;

    /**
     * An integer with position i in this array gives the position of the
     * corresponding field (field number i of the TupleStructure of this
     * QueryNode) in the array made of the concatenation of the array of fields
     * from all children of this QueryNode.
     */
    public int[] getFieldIndexes();

    public TupleStructure getTupleStructure();
}
