/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.type.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.lib.PTypeSpacePAAH;
import org.objectweb.medor.api.TypingException;

/**
 * This class extends the PTypeSpacePAAH of JORM for taking into account the
 * PType of Medor, and namely the PType of tuples.
 */
public class PTypeSpaceMedor extends PTypeSpacePAAH {

    public final static PType NAMING_CONTEXT = new QType(
        QType.TYPECODE_NAMING_CONTEXT,
        "PNamingContext",
        "org.objectweb.jorm.naming.api.PNamingContext",
        "PTypeSpaceMedor.NAMING_CONTEXT");

    public final static PType PNAME = new QType(
        QType.TYPECODE_PNAME,
        "PName",
        "org.objectweb.jorm.naming.api.PName",
        "PTypeSpaceMedor.PNAME");

	public final static PType OBJECT = new QType(
        QType.TYPECODE_OBJECT,
        "Object",
        "java.lang.Object",
        "PTypeSpaceMedor.OBJECT");

	public final static PType COLLECTION = new QType(
        QType.TYPECODE_COLLECTION,
        "Collection",
        "java.util.Collection",
        "PTypeSpaceMedor.COLLECTION");


	public PTypeSpaceMedor() {
		super();
	}

	/**
	 * This method tells if a given Type is an arithmetic type
	 * @param t The PType to be checked.
	 * @return true if the tested PType is an arithmetic type.
	 */
	public static final boolean isArithmeticType(PType t) {
		switch (t.getTypeCode()) {
		case QTypeTuple.TYPECODE_BYTE:
			return true;
		case QTypeTuple.TYPECODE_SHORT:
			return true;
		case QTypeTuple.TYPECODE_INT:
			return true;
		case QTypeTuple.TYPECODE_LONG:
			return true;
		case QTypeTuple.TYPECODE_FLOAT:
			return true;
		case QTypeTuple.TYPECODE_DOUBLE:
			return true;
		default:
			return false;
		}
	}

	/**
	 * This method returns the type which would result from applying a binary
	 * operator to 2 objects of given PType.
	 * If the two PTypes are incompatible, a TypingException is thrown.
	 * @param type1 The PType of the first object.
	 * @param type2 The PType of the second object.
	 * @return The PType of the result.
	 */
	public static PType getResultType(PType type1, PType type2)
		throws TypingException {
		if (type1.isa(type2)) {
			return type2;
		}
		else if (type2.isa(type1)) {
			return type1;
		}
		else {
			throw new TypingException("Incompatible types");
		}
	}

}
